#include <pwd.h>
#include <sys/types.h>
#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h>

#include "vt.h"

#define LINES 16

typedef struct
{
	char *code, *name;
} vt_command;

vt_command vcmd[] =
{
{"", 	"Exit"},
{"",	"Text"},
{"",	"RT_Text"},
{"",	"File"},
{"[0m", "Attributes off"},
{"[1m", "Bold on"},
{"[4m", "Underline on"},
{"[5m", "Blink on"},
{"[7m", "Reverse on"},
{"[2J", "Erase screen"},
{"[?3h", "132 col. mode"},
{"[?3l", "80 col. mode"},
{"[?4h", "Smooth scrolling"},
{"[?4l", "Jumpy scrolling"},
{"[?5h", "Reverse screen"},
{"[?5l", "Normal screen"},
{"c", 	"Reset"},
{"[!p", "Soft reset"},
{"#8", "EEEEEEEEEEEEE"},
{"}1*", "*************"},
{"[1;y", "Power up test"},
{"}2", "Video attribute test"},
{"}3", "Charset test"},
{"[2h", "Lock keyboard"},
{"[2l", "Unlock keyboard"},
{"[?75h", "Screen on"},
{"[?75l", "Screen off"},
};

void printmenu()
{
	int i;
	printf("%sCommands:%s\n", VT_BOLD, VT_NORMAL);
//	for (i=0; i<sizeof(vcmd)/sizeof(vt_command); i++)
	for (i=0; i<LINES; i++)
	{
		printf("%s%2d%s: %20s",VT_BOLD,i,VT_NORMAL,vcmd[i].name);
		if (i+LINES<(sizeof(vcmd)/sizeof(vt_command)))
			printf("\t%s%2d%s: %20s\n",VT_BOLD,i+LINES,VT_NORMAL,vcmd[i+LINES].name);
		else
			printf("\n");
	}
}

void sendcmd(FILE *f, int cmd)
{
	printf("Sending '%s'...\n", vcmd[cmd].name);
	fprintf(f,"\x1B%s", vcmd[cmd].code);
}

void main(int argc, char **argv)
{
	struct passwd *pw;
	struct stat buf;
	FILE *f,*f2;
	char s[256], c;
	int cmd;

	if (argc<2)
	{
		fprintf(stderr, "vth: terminal required.\n");
		return;
	}
	sprintf(s, "%s", argv[1]);
	if (stat(s, &buf)==-1)
	{
		fprintf(stderr, "vth: stat returned -1.\n");
		return;
	}

	pw = getpwuid(buf.st_uid);
	if (pw!=NULL)
		printf("vth: %s is owned by %s (%s, UID %d).\n", s, pw->pw_gecos, pw->pw_name, buf.st_uid);
	else
		printf("vth: %s is owned by UID %d.\n", s, buf.st_uid);

	f = fopen(s, "w");
	if (f==NULL)
	{
		fprintf(stderr, "vth: fopen returned NULL.\n");
		return;
	}
	do
	{
		printmenu();
		printf(">");
		scanf("%d",&cmd);
		printf("\n");
		switch (cmd)
		{
		case 0: break;
		case 1:
			printf("text: ");
			scanf("%s",s);
			fprintf(f,s);
			printf("\n");
		break;
		case 2:
			printf("rt_text ('_' to end): ");
			while ((c=fgetc(stdin))!='_')
			{
				fprintf(f,"%c",c);
				fflush(f);
			}
			printf("\n");
		break;
		case 3:
			printf("file: ");
			scanf("%s",s);
			f2 = fopen(s, "r");
			if (f2)
			{
				printf("Sending '%s'...\n", s);
				do
					fprintf(f, "%c", fgetc(f2));
				while(!feof(f2));
				fclose(f2);
			} else printf("Bad file.\n");
		break;
		default:
			sendcmd(f,cmd);
		break;
		}
	} while((cmd!=0)&&(cmd<sizeof(vcmd)/sizeof(vt_command)));
	fclose(f);
	printf("vth: clean exit.\n");
}
